package com.ycwl.basic.service.impl.mobile;

import cn.hutool.core.bean.BeanUtil;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.ycwl.basic.biz.OrderBiz;
import com.ycwl.basic.constant.BaseContextHandler;
import com.ycwl.basic.mapper.*;
import com.ycwl.basic.model.jwt.JwtInfo;
import com.ycwl.basic.model.mobile.goods.GoodsPageVO;
import com.ycwl.basic.model.mobile.order.IsBuyRespVO;
import com.ycwl.basic.model.mobile.scenic.ScenicAppVO;
import com.ycwl.basic.model.mobile.scenic.ScenicDeviceCountVO;
import com.ycwl.basic.model.mobile.scenic.ScenicIndexVO;
import com.ycwl.basic.model.mobile.scenic.account.ScenicLoginReq;
import com.ycwl.basic.model.mobile.scenic.account.ScenicLoginRespVO;
import com.ycwl.basic.model.mobile.scenic.content.ContentPageVO;
import com.ycwl.basic.model.pc.face.resp.FaceRespVO;
import com.ycwl.basic.model.pc.scenic.entity.ScenicAccountEntity;
import com.ycwl.basic.model.pc.scenic.entity.ScenicConfigEntity;
import com.ycwl.basic.model.pc.scenic.req.ScenicReqQuery;
import com.ycwl.basic.model.pc.scenic.resp.ScenicRespVO;
import com.ycwl.basic.model.pc.source.req.SourceReqQuery;
import com.ycwl.basic.model.pc.source.resp.SourceRespVO;
import com.ycwl.basic.model.pc.video.entity.MemberVideoEntity;
import com.ycwl.basic.model.pc.video.resp.VideoRespVO;
import com.ycwl.basic.repository.ScenicRepository;
import com.ycwl.basic.service.mobile.AppScenicService;
import com.ycwl.basic.repository.TemplateRepository;
import com.ycwl.basic.utils.ApiResponse;
import com.ycwl.basic.utils.JwtTokenUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @Author：longbinbin
 * @Date：2024/12/6 10:23
 */
@Slf4j
@Service
public class AppScenicServiceImpl implements AppScenicService {

    @Autowired
    private ScenicMapper scenicMapper;
    @Autowired
    private DeviceMapper deviceMapper;
    @Autowired
    private FaceMapper faceMapper;
    @Autowired
    private SourceMapper sourceMapper;
    @Autowired
    private TaskMapper taskMapper;
    @Autowired
    private VideoMapper videoMapper;
    @Autowired
    private TemplateMapper templateMapper;
    @Autowired
    private ScenicAccountMapper scenicAccountMapper;
    @Autowired
    private JwtTokenUtil jwtTokenUtil;

    @Value("${face.score}")
    private BigDecimal faceScore;
    @Autowired
    private TemplateRepository templateRepository;
    @Autowired
    private OrderBiz orderBiz;
    @Autowired
    private ScenicRepository scenicRepository;

    @Override
    public ApiResponse<PageInfo<ScenicAppVO>> pageQuery(ScenicReqQuery scenicReqQuery) {
        PageHelper.startPage(scenicReqQuery.getPageNum(), scenicReqQuery.getPageSize());
        List<ScenicAppVO> list = scenicMapper.appList(scenicReqQuery);
        PageInfo<ScenicAppVO> pageInfo = new PageInfo<>(list);
        return ApiResponse.success(pageInfo);
    }

    @Override
    public ApiResponse<ScenicDeviceCountVO> deviceCountByScenicId(Long scenicId) {
        JwtInfo worker = JwtTokenUtil.getWorker();
        ScenicDeviceCountVO scenicDeviceCountVO = deviceMapper.deviceCountByScenicId(scenicId, worker.getUserId());
        return ApiResponse.success(scenicDeviceCountVO);
    }

    @Override
    public ApiResponse<ScenicRespVO> getDetails(Long id) {
        ScenicRespVO scenicRespVO = scenicMapper.getAppById(id);
        ScenicDeviceCountVO scenicDeviceCountVO = deviceMapper.deviceCountByScenicId(id, -1L);
        scenicRespVO.setLensNum(scenicDeviceCountVO.getTotalDeviceCount());
        return ApiResponse.success(scenicRespVO);
    }

    @Override
    public ApiResponse<List<ScenicAppVO>> scenicList(ScenicIndexVO scenicIndexVO) {
        List<ScenicAppVO> dataList = scenicMapper.scenicListByLnLa(scenicIndexVO);
        return ApiResponse.success(dataList);
    }

    @Override
    public ApiResponse<List<ContentPageVO>> faceContentList(Long userId, Long faceId) {
        FaceRespVO faceRespVO = faceMapper.getById(faceId);
        if (faceRespVO == null) {
            return ApiResponse.success(new ArrayList<>());
        }
        if (StringUtils.isBlank(faceRespVO.getMatchSampleIds())) {
            return ApiResponse.success(new ArrayList<>());
        }
        List<ContentPageVO> contentList = templateMapper.listFor(faceRespVO.getScenicId());
        contentList.forEach(contentPageVO -> {
            List<MemberVideoEntity> memberVideoEntityList = videoMapper.userFaceTemplateVideo(userId, faceId, contentPageVO.getTemplateId());
            contentPageVO.setContentType(1);
            if (!memberVideoEntityList.isEmpty()) {
                contentPageVO.setIsBuy(memberVideoEntityList.get(0).getIsBuy());
                contentPageVO.setContentId(memberVideoEntityList.get(0).getVideoId());
                VideoRespVO videoMapperById = videoMapper.getById(contentPageVO.getContentId());
                contentPageVO.setDuration(videoMapperById.getDuration());
                contentPageVO.setLockType(-1);
            } else {
                contentPageVO.setContentType(0);
                boolean canGenerate = templateRepository.determineTemplateCanGenerate(contentPageVO.getTemplateId(), faceId);
                if (canGenerate) {
                    contentPageVO.setLockType(0);
                } else {
                    contentPageVO.setLockType(1);
                }
            }
        });

        SourceReqQuery sourceReqQuery = new SourceReqQuery();
        sourceReqQuery.setScenicId(faceRespVO.getScenicId());
        sourceReqQuery.setFaceId(faceId);
        sourceReqQuery.setMemberId(userId);
        //查询源素材
        List<SourceRespVO> sourceList = sourceMapper.queryByRelation(sourceReqQuery);
        ScenicConfigEntity scenicConfig = scenicRepository.getScenicConfig(faceRespVO.getScenicId());
        sourceList.stream().collect(Collectors.groupingBy(SourceRespVO::getType)).keySet()
                .stream()
                .filter(type -> {
                    if (Integer.valueOf(1).equals(type)) {
                        if (Integer.valueOf(1).equals(scenicConfig.getDisableSourceVideo())) {
                            return false;
                        }
                    } else if (Integer.valueOf(2).equals(type)) {
                        if (Integer.valueOf(1).equals(scenicConfig.getDisableSourceImage())) {
                            return false;
                        }
                    }
                    return true;
                })
                .forEach(type -> {
                    ContentPageVO contentPageVO = new ContentPageVO();
                    if (type == 1) {
                        contentPageVO.setName("原片集");
                    } else {
                        contentPageVO.setName("照片集");
                    }
                    IsBuyRespVO isBuyRespVO = orderBiz.isBuy(userId, faceRespVO.getScenicId(), type, faceId);
                    contentPageVO.setSourceType(isBuyRespVO.getGoodsType());
                    contentPageVO.setContentId(isBuyRespVO.getGoodsId());
                    if (isBuyRespVO.isBuy()) {
                        contentPageVO.setIsBuy(1);
                    } else {
                        contentPageVO.setIsBuy(0);
                    }
                    contentPageVO.setScenicId(faceRespVO.getScenicId());
                    contentPageVO.setTemplateCoverUrl(sourceList.get(0).getUrl());
                    contentPageVO.setContentType(2);
                    contentList.add(contentPageVO);
                });

        return ApiResponse.success(contentList);
    }

    @Override
    public ApiResponse<ScenicLoginRespVO> login(ScenicLoginReq scenicLoginReq) throws Exception {

        ScenicAccountEntity scenicAccount = scenicAccountMapper.getByAccount(scenicLoginReq.getAccount());
        if (scenicAccount == null) {
            return ApiResponse.fail("账号不存在");
        }
        if (scenicAccount.getStatus() == 0) {
            return ApiResponse.fail("账号已被禁用");
        }
        if (!scenicAccount.getPassword().equals(scenicLoginReq.getPassword())) {
            return ApiResponse.fail("密码错误");
        }

        JwtInfo jwtInfo = new JwtInfo();
        jwtInfo.setName(scenicAccount.getName());
        jwtInfo.setAccount(scenicAccount.getAccount());
        jwtInfo.setUserId(scenicAccount.getId());
        jwtInfo.setScenicId(scenicAccount.getScenicId());
        String token = jwtTokenUtil.generateToken(jwtInfo);

        ScenicLoginRespVO scenicLoginRespVO = new ScenicLoginRespVO();
        BeanUtil.copyProperties(scenicAccount,scenicLoginRespVO);
        scenicLoginRespVO.setToken(token);
        return ApiResponse.success(scenicLoginRespVO);
    }

    @Override
    public ApiResponse<List<ContentPageVO>> contentListUseDefaultFace() {
        FaceRespVO lastFaceByUserId = faceMapper.findLastFaceByUserId(BaseContextHandler.getUserId());
        return faceContentList(lastFaceByUserId.getMemberId(), lastFaceByUserId.getId());
    }

    @Override
    public ApiResponse<ScenicRespVO> getMyScenic() {
        String userId = BaseContextHandler.getUserId();
        ScenicAccountEntity account = scenicAccountMapper.findAccountById(userId);
        if (account == null) {
            return ApiResponse.fail("用户未绑定景区");
        }
        return getDetails(account.getScenicId());
    }
}
